<?php

namespace Conformity\Common\ConfigStore;

class JsonConfigStore extends AbstractConfigStore
{
    private array $configuration = [];

    private string $filename;

    private bool $readOnly = false;

    private int $fileTime = 0;

    private int $fileSize = 0;

    public function __construct(string $filename, bool $readOnly=false)
    {
        $this->filename = $filename;
        if (file_exists($filename) && !is_writable($filename)) {
            $this->readOnly = true;
        }

        $this->readConfig();
    }

    protected function readConfig(): void
    {
        if (!file_exists($this->filename)) {
            $this->fileTime = 0;
            $this->fileSize = 0;
            return;
        }
        $config = json_decode(file_get_contents($this->filename), true);
        if (is_array($config)) {
            $this->configuration = $config;
            $this->fileTime = filemtime($this->filename);
            $this->fileSize = filesize($this->filename);
        } else {
            // log error...
        }
    }

    protected function writeConfig(): void
    {
        assert("\$this->readOnly === false");

        $json = json_encode($this->configuration);
        $fd = fopen($this->filename, "wb");
        fwrite($fd, $json);
        fclose($fd);
    }

    public function has(string $key): bool
    {
        $items = explode("/", trim($key, "/"));
        $p = $this->configuration; // pointer
        foreach ($items as $item) {
            if (!array_key_exists($item, $p)) {
                return false;
            }
            $p = $p[$item];
        }
        return true;
    }

    public function get(string $key, mixed $default = null): mixed
    {
        $items = explode("/", trim($key, "/"));
        $p = $this->configuration; // pointer
        foreach ($items as $item) {
            if (!array_key_exists($item, $p)) {
                return $default;
            }
            $p = $p[$item];
        }
        return $p;
    }

    public function set(string $key, mixed $value): void
    {
        $items = explode("/", trim($key, "/"));
        $k = array_pop($items);
        $p =& $this->configuration; // pointer
        foreach ($items as $item) {
            if (!array_key_exists($item, $p)) {
                $p[$item] = [];
            }
            $p =& $p[$item];
        }
        if (is_array($p))
            $p[$k] = $value;
        else
            $p = [$k=>$value];
        // apply 
        if (!$this->readOnly) {
            $this->writeConfig();
        }
    }

    public function unset(string $key): void
    {
        $items = explode("/", trim($key, "/"));
        $k = array_pop($items);
        $p =& $this->configuration; // pointer
        foreach ($items as $item) {
            if (!array_key_exists($item, $p)) {
                $p[$item] = [];
            }
            $p =& $p[$item];
        }
        if (is_array($p))
            unset($p[$k]);

        // apply 
        if (!$this->readOnly) {
            $this->writeConfig();
        }
    }
}

