<?php

namespace Conformity\Common\System;

class HostId
{
    private string $hostId;

    public function __construct()
    {
        if ($hostId = getenv("CF_HOSTID")) {
            $this->hostId = $hostId;
        } else {
            $this->generateHostId();
        }
    }

    private function generateHostId()
    {
        $machineId = $this->getMachineId();

        $this->hostId = md5($machineId);
    }

    private function getMachineId(): string
    {
        if (file_exists("/etc/machine-id")) {
            return trim(file_get_contents("/etc/machine-id"));
        } elseif (file_exists("/var/lib/dbus/machine-id")) {
            return trim(file_get_contents("/var/lib/dbus/machine-id"));
        } elseif (file_exists("/var/db/dbus/machine-id")) {
            return trim(file_get_contents("/var/db/dbus/machine-id"));
        } else {
            foreach (glob("/sys/class/net/*/address") as $file) {
                $addr = trim(file_get_contents($file));
                if ($addr && $addr != "00:00:00:00:00:00") {
                    return $addr.":".gethostname();
                }
            }
            throw new \RuntimeException("Unable to resolve usable machine-id");
        }
    }

    public function getHostId(): string
    {
        return $this->hostId;
    }

    public function __toString(): string
    {
        return $this->getHostId();
    }
}
