<?php

namespace Conformity\Common\ConfigStore;


class JsonConfigStoreTest extends \PHPUnit\Framework\TestCase
{

    public function setUp(): void
    {
        file_exists("/tmp/configstoretest.json") && unlink("/tmp/configstoretest.json");
    }

    public function tearDown(): void
    {
        file_exists("/tmp/configstoretest.json") && unlink("/tmp/configstoretest.json");
    }

    /**
     * @covers JsonConfigStore::set
     * @covers JsonConfigStore::get
     */
    public function testWritingNodes()
    {
        $store = new JsonConfigStore("/tmp/configstoretest.json");

        $store->set("foo/bar/baz", 42);
        $this->assertEquals(42, $store->get("foo/bar/baz"));
        $this->assertEquals(['baz'=>42], $store->get("foo/bar"));
        $this->assertEquals(['bar'=>['baz'=>42]], $store->get("foo"));

        $store->set("foo/bar/bin", true);
        $this->assertEquals(42, $store->get("foo/bar/baz"));
        $this->assertEquals(true, $store->get("foo/bar/bin"));
        $this->assertEquals(['baz'=>42,'bin'=>true], $store->get("foo/bar"));
        $this->assertEquals(['bar'=>['baz'=>42,'bin'=>true]], $store->get("foo"));
    }

    /**
     * @covers JsonConfigStore::set
     * @covers JsonConfigStore::get
     * @covers JsonConfigStore::unset
     */
    public function testUnsettingKeys()
    {
        $store = new JsonConfigStore("/tmp/configstoretest.json");

        $store->set("foo/bar/baz", 42);
        $store->set("foo/bar/bin", true);

        $store->unset("foo/bar/baz");
        $this->assertEquals(['bar'=>['bin'=>true]], $store->get("foo"));
        $store->unset("foo/bar");
        $this->assertEquals([], $store->get("foo"));

    }

    /**
     * @covers JsonConfigStore::set
     * @covers JsonConfigStore::get
     */
    public function testArrayOperations()
    {
        $store = new JsonConfigStore("/tmp/configstoretest.json");

        $store->set("numbers", [ 4.2, 42 ]);
        $this->assertEquals([ 4.2, 42 ], $store->get("numbers"));
        $store->set("numbers", [ 4.2, 42, 420 ]);
        $this->assertEquals([ 4.2, 42, 420 ], $store->get("numbers"));
    }

    /**
     * @covers JsonConfigStore::writeConfig
     * @covers JsonConfigStore::readConfig
     */
    public function testPersistingStore()
    {
        $store = new JsonConfigStore("/tmp/configstoretest.json");
        $store->set("foo/bar/baz", 42);
        $this->assertEquals(42, $store->get("foo/bar/baz"));
        $this->assertEquals(['baz'=>42], $store->get("foo/bar"));
        $this->assertEquals(['bar'=>['baz'=>42]], $store->get("foo"));
        $store = null;

        $store = new JsonConfigStore("/tmp/configstoretest.json");
        $this->assertEquals(42, $store->get("foo/bar/baz"));
        $this->assertEquals(['baz'=>42], $store->get("foo/bar"));
        $this->assertEquals(['bar'=>['baz'=>42]], $store->get("foo"));
        $store = null;

    }

}